/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};


/*
 *  Boilerplate code for encoding and decoding data transmitted to / received from TI devices.
 */

/**
 * This code implements the IPacketCodec interface.
 *
 * @constructor
 * @implements gc.databind.IPacketCodec
 */
CustomCodec = function() {};
CustomCodec.prototype.rxPktCtr = 0;
CustomCodec.prototype.rxMsgCtr = 0;
CustomCodec.prototype.txPktCtr = 0;
CustomCodec.prototype.txMsgCtr = 0;
CustomCodec.prototype.strRxBuffer = "";
CustomCodec.prototype.strTxBuffer = "";
CustomCodec.prototype.isConnected = false;
CustomCodec.prototype.initComplete = false;
CustomCodec.prototype.templateObj = undefined;

/**
 * Encodes data into a packet for sending to the target device.
 *
 * @param target - function to call to pass on the encoded data towards the target
 * @param data - The object containing the binding properties from the GUI that is to be encoded into a packet for sending.
 */
CustomCodec.prototype.encode = function(target, data) {
    this.txPktCtr++;

   // The following example code shows how to encode property values from the GUI into
   // a JSON formatted string.  You would use this if you wanted to intercept data from the GUI
   // and rework it before passing it on to the target device.
/*
   try{
        this.txMsgCtr++;
        if (this.txMsgCtr % 256){
            console.log("Number of tx messages to target: "+this.txMsgCtr);
        }
        var strToSend = JSON.stringify(data) +"\n";
        // send the string to the target
        target(strToSend);
    }
    catch(ex){
        console.log("CustomCodec.encode (to target): exception="+ex);
    }

*/

};
/**
 * Decodes packets from the target device into data objects.  One object for each packet of data.
 * Unless you have chained your custom codec with the CR codec, the packet data is not framed,
 * so this method is responsible for decoding partial packets as well as multiple packets.  Partial packets
 * should not be returned, but deferred until the next decode method call with more raw data to process.  As a
 * result, this method may return 0 or more packets of data on each invocation.
 *
 * @param target -  function to call to pass on the encoded data towards the GUI
 * @param data  - The raw data received that needs to be decoded from packets into objects.
 * @return - true if connected (i.e. valid data received), false if not connected
 */
CustomCodec.prototype.decode = function(target, data) {
    this.isConnected = true;
    this.rxPktCtr++;

    // The following example code shows how to decode JSON strings coming up from the target
    // into a Javascript object.  You would use this if you wanted to intercept data from the target
    // and rework it before passing it on to the GUI for binding

    var strRxData = data.map(function(value) {
        return String.fromCharCode(value);
    }).join("");
    this.strRxBuffer = this.strRxBuffer + strRxData;
    var strMessage = '';
    while (this.strRxBuffer.indexOf('\n') !== -1) {
        strMessage = this.strRxBuffer.substring(0, this.strRxBuffer.indexOf('\n') + 1);
        this.strRxBuffer = this.strRxBuffer.substring(strMessage.length);
        // Rework the following code if your codec is not receiving Javascript objects from the GUI
        try {
            var jsonObj = JSON.parse(strMessage);
            this.rxMsgCtr++;
            if (this.rxMsgCtr % 256) {
                console.log("Number of tx messages from target: " + this.rxMsgCtr);
            }
            var newArray = [jsonObj.analog[0], jsonObj.analog[1], jsonObj.analog[2]];
            jsonObj.barGraphData = newArray.join(';');
            // send the received and decoded javascript object to the application where you can use it to bind to widget properties
            target(jsonObj);
        } catch (ex) {
            console.log("CustomCodec.decode (from target): strMessage=" + strMessage + ", exception=" + ex);
        }
    }


    // Return true to set the 'Hardware Connected' status in the status bar
    return this.isConnected;

};

CustomCodec.prototype.close = function() {
    gc.databind.unregisterCustomCodec("custom", CustomCodec);
    CustomCodec.initComplete = false;
    CustomCodec.isConnected = false;
};

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    // You can 'chain' codecs together to make it easier to work with the data
    // coming from the device or going to the device.
    // The following codecs are provided:
    //   CR  - use this if the target data uses /n as a delimiter for framing.
    //         The CR codec will take care of buffering the incoming data so that
    //         your codec will only receive complete 'ready to use' packets
    //  If your target uses /n delimited strings, set baseCodecs = "CR"
    //  If your target uses some other format for data (e.g. binary data or custom strings) leave baseCodecs = "undefined"

    var baseCodecs;
    var gcDatabindInitComplete = false;
    var templateInitComplete = false;
    console.log("custom codec: about to register..."); // log a message that you will be able to see in the debugger
    if ((gc) && (gc.databind)) {
        gc.databind.registerCustomCodec("custom", CustomCodec, baseCodecs);
    } else {
        document.addEventListener('gc-databind-ready', function() {
            gc.databind.registerCustomCodec("custom", CustomCodec, baseCodecs);
            gcDatabindInitComplete = true;
            if (templateInitComplete) {
                CustomCodec.initComplete = true;
            }
        });
    }

    CustomCodec.templateObj = document.querySelector('#template_obj');
    if (CustomCodec.templateObj) {
        // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
        // before trying to access components in the application.
        CustomCodec.templateObj.addEventListener('dom-change', function() {
            if (CustomCodec.initComplete) return;
            CustomCodec.templateObj.async(function() {
                templateInitComplete = true;
                if (gcDatabindInitComplete) {
                    CustomCodec.initComplete = true;
                }
                console.log("Application template has been stamped.");
                // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
                // e.g. to access a widget with an id of 'widget_id' you can use CustomCodec.templateObj.$.widgetId

            }, 1);
        });
    } else {
        templateInitComplete = true;
        if (gcDatabindInitComplete) {
            CustomCodec.initComplete = true;
        }
        console.log("CustomCodec: Failed to find id template_obj in the Application!");
    }

};

if (document.querySelector('#template_obj')) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init.bind(this))
}